package mooctest.org.backend.site.controller;

import lombok.extern.slf4j.Slf4j;
import mooctest.org.backend.site.command.NotificationCommand;
import mooctest.org.backend.site.data.vo.NotificationVO;
import mooctest.org.backend.site.exception.BaseException;
import mooctest.org.backend.site.service.NotificationService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.cloud.context.config.annotation.RefreshScope;
import org.springframework.validation.BindingResult;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * @author guochao
 * @date 2019-12-18 16:43
 */
@RefreshScope
@RestController
@Slf4j
public class NotificationController {

    @Value("${username:lily}")
    private String username;

    @RequestMapping("/username")
    public String get() {
        return username;
    }

    @Autowired
    private NotificationService notificationService;

    /**
     * 查看首页大赛通知
     * @return
     */
    @ResponseBody
    @RequestMapping(value = "/api/view/index", method = RequestMethod.GET)
    public List<NotificationVO> getIndexNotification(){
        log.info("获取大赛首页通知列表");
        List<NotificationVO> notificationDTOList = notificationService.getIndexNotification();
        return notificationDTOList;
    }

    /**
     * 查看所有大赛通知
     * @return
     */
    @ResponseBody
    @RequestMapping(value = "/api/view", method = RequestMethod.GET)
    public List<NotificationVO> getAllNotification(){
        log.info("获取大赛通知列表");
        List<NotificationVO> notificationDTOList = notificationService.getAllNotification();
        return notificationDTOList;
    }

    /**
     * 查看指定大赛通知
     * @param id
     * @return
     */
    @ResponseBody
    @RequestMapping(value = "/api/view/{id}", method = RequestMethod.GET)
    public NotificationVO getNotification(@PathVariable(value = "id") Long id){
        NotificationVO notificationDTO = notificationService.getNotificationById(id);
        return notificationDTO;
    }

    /**
     * 校验密码
     * @return
     */
    @RequestMapping(value = "/api/checkPWD/{password}", method = RequestMethod.GET)
    public boolean checkPassword(@PathVariable("password") String password){
        return notificationService.checkPassword(password);
    }

    /**
     * 创建大赛通知
     * @param notificationCommand
     * @param result
     * @return
     */
    @RequestMapping(value = "/api/create", method = RequestMethod.POST)
    public NotificationVO createNotification(@Validated @RequestBody NotificationCommand notificationCommand, BindingResult result){
        if (result.hasErrors()) {
            throw new BaseException(result.getFieldErrors().toString());
        }
        return notificationService.createNotification(notificationCommand);
    }

    /**
     * 更新指定大赛通知
     * @param notificationCommand
     * @param result
     * @return
     */
    @RequestMapping(value = "/api/update/{notificationId}", method = RequestMethod.PUT)
    public NotificationVO updateNotification(@Validated @RequestBody NotificationCommand notificationCommand, @PathVariable("notificationId") Long notificationId, BindingResult result){
        if (result.hasErrors()) {
            throw new BaseException(result.getFieldErrors().toString());
        }
        return notificationService.updateNotification(notificationCommand, notificationId);
    }

    /**
     * 删除指定大赛通知
     * @return
     */
    @RequestMapping(value = "/api/delete/{notificationId}", method = RequestMethod.DELETE)
    public Boolean deletedNotification(@PathVariable("notificationId") Long notificationId){
        return notificationService.deleteNotification(notificationId);
    }
}
