package mooctest.org.backend.site.reposity;

import mooctest.org.backend.site.dao.NotificationDao;
import mooctest.org.backend.site.data.DeletedStatus;
import mooctest.org.backend.site.domainObject.Notification;
import mooctest.org.backend.site.exception.BaseException;
import mooctest.org.backend.site.model.NotificationPO;
import mooctest.org.backend.site.utils.Converter;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;

/**
 * @author guochao
 * @date 2019-12-18 17:36
 */
@Component
public class NotificationRepo implements INotificationRepo{

    @Autowired
    private NotificationDao notificationDao;

    @Override
    public List<Notification> getAllNotification(){
        return notificationDao.findAll(DeletedStatus.NOT_DELETED).stream().map(notificationPO -> Converter.convert(Notification.class, notificationPO)).collect(Collectors.toList());
    }

    @Override
    public List<Notification> getIndexNotification(int limitCount){
        return notificationDao.findOrderByIdDescLimit(DeletedStatus.NOT_DELETED, limitCount).stream().map(notificationPO -> Converter.convert(Notification.class, notificationPO)).collect(Collectors.toList());
    }

    @Override
    public List<Notification> getNotificationByType(int type){
        return notificationDao.findAllByType(type, DeletedStatus.NOT_DELETED).stream().map(notificationPO -> Converter.convert(Notification.class, notificationPO)).collect(Collectors.toList());
    }

    @Override
    public Notification getNotificationById(Long id) {
        Optional<NotificationPO> notificationOptional = notificationDao.findByIdAndDeletedStatus(id, DeletedStatus.NOT_DELETED);
        if(!notificationOptional.isPresent()){
            throw new BaseException("无此通知的信息!");
        }
        return Converter.convert(Notification.class, notificationOptional.get());
    }

    @Override
    public Long saveNotification(Notification notification) {
        return notificationDao.save(Converter.convert(NotificationPO.class, notification)).getId();
    }

}
