package mooctest.org.backend.site.service;

import lombok.extern.slf4j.Slf4j;
import mooctest.org.backend.site.command.NotificationCommand;
import mooctest.org.backend.site.data.DeletedStatus;
import mooctest.org.backend.site.data.NotificationType;
import mooctest.org.backend.site.data.vo.NotificationVO;
import mooctest.org.backend.site.domainObject.Notification;
import mooctest.org.backend.site.reposity.INotificationRepo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.stream.Collectors;

/**
 * @author guochao
 * @date 2019-12-18 17:36
 */
@Service
@Slf4j
public class NotificationService {

    @Autowired
    private INotificationRepo notificationRepo;
    @Value("${password}")
    private String correctPassword;
    @Value("${limit.count}")
    private String limitCount;

    public List<NotificationVO> getIndexNotification(){
        return notificationRepo.getIndexNotification(Integer.parseInt(limitCount))
                .stream().map(notification -> new NotificationVO(notification)).collect(Collectors.toList());
    }

    public List<NotificationVO> getAllNotification(){
        return notificationRepo.getAllNotification()
                .stream().map(notification -> new NotificationVO(notification)).collect(Collectors.toList());
    }

    public NotificationVO getNotificationById(Long id) {
        Notification notification = notificationRepo.getNotificationById(id);
        log.info("获取id为：{} title为：\"{}\" 的大赛通知", id, notification.getTitle());
        return new NotificationVO(notification);
    }

    public NotificationVO createNotification(NotificationCommand notificationCommand) {
        Notification notification = notificationCommand.toNotification();
        notification.createNotification();
        Long id = notificationRepo.saveNotification(notification);
        notification.setId(id);
        log.info("创建id为：{} title为：\"{}\" 的大赛通知", id, notification.getTitle());
        checkAndSaveNotification(notificationCommand, notification);
        return new NotificationVO(notification);
    }

    public NotificationVO updateNotification(NotificationCommand notificationCommand, Long notificationId) {
        Notification notification = notificationRepo.getNotificationById(notificationId);
        log.info("修改id为：{} title为：\"{}\" 的大赛通知", notificationId, notificationCommand.getTitle());
        notification.update(notificationCommand);
        checkAndSaveNotification(notificationCommand, notification);
        notificationRepo.saveNotification(notification);
        return new NotificationVO(notification);
    }

    /**
     * 保存通知信息，区分是否置顶，保证创建的置顶通知仅有1个
     * @param notificationCommand
     * @param notification
     */
    private void checkAndSaveNotification(NotificationCommand notificationCommand, Notification notification) {
        // 置顶操作
        if(notificationCommand.getType() == NotificationType.TOP){
            notificationRepo.getNotificationByType(notificationCommand.getType())
                    .stream().filter(notificationType -> notificationType.getDeletedStatus() == DeletedStatus.NOT_DELETED && notificationType.getId() != notification.getId())
                    .map(notificationType -> {
                        notificationType.changeTypeToNew();
                        notificationRepo.saveNotification(notificationType);
                        return notificationType;
                    }).collect(Collectors.toList());
        }
        notificationRepo.saveNotification(notification);
    }

    public Boolean deleteNotification(Long notificationId) {
        Notification notification = notificationRepo.getNotificationById(notificationId);
        log.info("删除id为：{} title为：\"{}\" 的大赛通知", notificationId, notification.getTitle());
        notification.delete();
        notificationRepo.saveNotification(notification);
        if(notification.getDeletedStatus() == DeletedStatus.IS_DELETED){
            return true;
        }else{
            return false;
        }

    }

    public boolean checkPassword(String password) {
        log.info("输入的密码为：{}", password);
        return correctPassword.equals(password);
    }
}
